#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#define eprintf(...) fprintf(stderr, __VA_ARGS__)

#define BUF_SIZE	129

static const char DEFAULT_DEVICE[] = "/dev/ttyUSB0";

static int open_comm(const char *device) {
	struct termios o;
	int dd;

	dd = open(device, O_WRONLY | O_NOCTTY);

	if (tcgetattr(dd, &o) < 0) goto err;

	cfsetispeed(&o, B9600);
	cfsetospeed(&o, B9600);

	o.c_cflag |= CLOCAL;
	o.c_cflag &= ~CREAD;	// no receiver
	o.c_cflag &= ~(PARENB | PARODD);	// no parity
	o.c_cflag &= ~CSTOPB;	// 1 stop bit
	o.c_cflag &= ~CSIZE;
	o.c_cflag |= CS8;	// 8 bit
	o.c_cflag |= CRTSCTS;	// hw handshaking
	o.c_cflag &= ~HUPCL;

	o.c_lflag = 0;
	//o.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);	// raw input

	o.c_iflag = IGNBRK | IGNPAR;
	o.c_iflag &= ~(IXON | IXOFF | IXANY);

	o.c_oflag = 0;

	if (tcsetattr(dd, TCSANOW, &o) < 0) goto err;

	return dd;

err:
	close(dd);
	return -1;
}

static void online(int sd, int on) {
	int i = TIOCM_DTR;
	if (ioctl(sd, on ? TIOCMBIS : TIOCMBIC, &i) < 0)
		perror("ioctl(dtr)");
	usleep(100 * 1000);
}

static void help(const char *pgm) {
	printf("Usage: %s -01nh [-d device]\n", pgm);
	puts(" -0 Go offline and exit");
	puts(" -1 Go online and exit");
	puts(" -n Replace \\n with \\r");
	puts(" -h Display this help and exit");
	puts(" -D Debug");
}

int main(int argn, char *argv[]) {
	const char *device = DEFAULT_DEVICE;
	unsigned char buf[24];
	int sd, l, o, retval = 0;
	signed char ool = 0, crlf = 0, debug = 0;

	while ((o = getopt(argn, argv, "01nhDd:")) != -1) {
		switch (o) {
			case 'D':
				debug = 1;
				break;
				
			case 'd':
				device = optarg;
				break;

			case 'n':
				crlf = 1;
				break;

			case '0':
				if (ool == 1) {
					eprintf("Online/offline conflict!\n");
					return 1;
				}
				ool = -1;
				break;

			case '1':
				if (ool == -1) {
					eprintf("Online/offline conflict!\n");
					return 1;
				}
				ool = 1;
				break;

			default:
				help(argv[0]);
				return o != 'h';
		}
	}

	// open
	sd = open_comm(device);
	if (sd < 0) {
		eprintf("Can't open device %s!\n", device);
		return 1;
	}

	// online/offline?
	if (ool) {
		online(sd, ool == 1);
		goto kraj;
	}

	// print
	online(sd, 1);
	while ((l = read(0, buf, sizeof(buf))) > 0) {
		int ll, u = 0;

		if (debug) {
			int i;
			fprintf(stderr, "Got:");
			for (i = 0; i < l; ++i)
				fprintf(stderr, " 0x%02X", buf[i]);
			fprintf(stderr, "\n");
		}

		// \n -> \r
		if (crlf) {
			int i;
			for (i = 0; i < l; ++i)
				if (buf[i] == '\n')
					buf[i]='\r';
		}

		do {
			if ((ll = write(sd, buf + u, l)) <= 0) {
				perror("write");
				retval = 1;
				goto kraj;
			}
			l -= ll;
			u += ll;
		} while(l);
	}

kraj:
	close(sd);
	return retval;
}
